#!/bin/bash

# Builds the anion and kations folders for the g(r). 
# The ion sigma and epsilon parameters are taken from the ipython notebook.

#set -e

module load gromacs/single/2019
BASE=$(realpath .)

# Lengths of all lists must be the same!
ANION="Cl"
ANION_SIGMAS=( 0.405 )
ANION_EPSILONS=( 0.627 )

KATION="Na"
KATION_SIGMAS=( 0.25 )
KATION_EPSILONS=( 0.2 )

NGPU=1
# Set watermodels
# 1: SPC      
# 2: SPC/E    
# 3: TIP3P     
# 4: TIP4P    
# 5: TIP4P/ε
WM_INDICES=( 3 )

# Submit to SLURM or run directly
SUBMIT=false

N=${#ANION_SIGMAS[@]}
N=$((N-1))

for WM_INDEX in ${WM_INDICES[@]}; do
if [ $WM_INDEX == 1 ]; then 
    WM="spc"
    WGRO="spc216.gro"
elif [ $WM_INDEX == 2 ]; then
    WM="spce"
    WGRO="spc216.gro"
elif [ $WM_INDEX == 3 ]; then
    WM="tip3p"
    WGRO="spc216.gro"
elif [ $WM_INDEX == 4 ]; then
    WM="tip4p"
    WGRO="tip4p.gro"
elif [ $WM_INDEX == 5 ]; then
    WM="tip4pe"
    WGRO="tip4p.gro"
else
    echo "$WM_INDEX is not valid. Should be between 1 and 5." > /dev/stderr
    exit 2
fi

if [ ${#ANION_SIGMAS[@]} != ${#ANION_EPSILONS[@]} ] || 
   [ ${#ANION_SIGMAS[@]} != ${#KATION_SIGMAS[@]} ] || 
   [ ${#ANION_SIGMAS[@]} != ${#KATION_EPSILONS[@]} ]; then
    echo "The parameter lists do not have the same length." > /dev/stderr
    exit 2
fi

mkdir -p $WM
cd $WM

mkdir -p $KATION$ANION
cd $KATION$ANION


for i in $(seq 0 $N); do
ANION_SIGMA=${ANION_SIGMAS[$i]}
ANION_EPSILON=${ANION_EPSILONS[$i]}

ANION_DIRNAME=$ANION\_sigma\_$ANION_SIGMA\_epsilon\_$ANION_EPSILON
mkdir -p $ANION_DIRNAME
cd $ANION_DIRNAME

KATION_SIGMA=${KATION_SIGMAS[$i]}
KATION_EPSILON=${KATION_EPSILONS[$i]}

KATION_DIRNAME=$KATION\_sigma\_$KATION_SIGMA\_epsilon\_$KATION_EPSILON
mkdir -p $KATION_DIRNAME
cd $KATION_DIRNAME

for CONC in 0.5 1 2 3 4; do

    mkdir -p $CONC
    cd $CONC
    pwd

    ln -sf $(realpath --relative-to="." "$BASE/..")/si.ff si.ff
    gmx -quiet solvate -box 6.5 6.5 6.5 -cs $WGRO
    gmx -quiet pdb2gmx -f out.gro << EOF
        1
        $WM_INDEX
EOF

    # Add custom ion parameters
    ANION_NUMBER=$(cat si.ff/ffnonbonded.itp | grep $ANION | awk '{print $2}')
    ANION_MASS=$(cat si.ff/atomtypes.atp | grep $ANION | awk '{print $2}')
    KATION_NUMBER=$(cat si.ff/ffnonbonded.itp | grep $KATION | awk '{print $2}')
    KATION_MASS=$(cat si.ff/atomtypes.atp | grep $KATION | awk '{print $2}')
    sed -i "/forcefield.itp/a [ atomtypes ]\n $KATION  $KATION_NUMBER  $KATION_MASS  0.0000 A  $KATION_SIGMA $KATION_EPSILON\n $ANION  $ANION_NUMBER  $ANION_MASS  0.0000 A  $ANION_SIGMA $ANION_EPSILON\n" topol.top

    gmx grompp -c conf.gro -f $BASE/input/em.mdp -o ion.tpr -maxwarn 2
    gmx genion -s ion.tpr -nname $ANION -pname $KATION -conc $CONC -rmin 0.3 -p topol.top << EOF
        2
EOF

    gmx -quiet grompp -quiet -f $BASE/input/em.mdp -c out.gro -o em.tpr -maxwarn 2
    gmx -quiet mdrun -quiet -deffnm em -nt 1

    gmx -quiet grompp -quiet -f $BASE/input/nvt.mdp -c em.gro -o nvt.tpr -maxwarn 2
    gmx -quiet mdrun -quiet -v -deffnm nvt $@

    gmx -quiet grompp -quiet -f $BASE/input/npt.mdp -c nvt.gro -o npt.tpr -maxwarn 2
    gmx -quiet mdrun -quiet -v -deffnm npt $@

    gmx -quiet grompp -quiet -f $BASE/input/grompp.mdp -c npt.gro -maxwarn 2

    sed -e "s/--job-name=*/--job-name=$WM\_$KATION_DIRNAME\_$ANION_DIRNAME\_c_$CONC/g" \
        -e "s/KATION/$KATION/g" \
        -e "s/NGPU/$NGPU/g" \
        -e "s/ANION/$ANION/g" $BASE/srun_template.sh > srun.sh

    rm \#*
    rm step*

    if $SUBMIT; then
        sbatch srun.sh
    else
        bash srun.sh
    fi

    cd ..
done
cd ../..
done
cd ../..
done
